#include "MotionDetection.h"

MotionDetection::MotionDetection(){//:handler(FSPI){
    handler = new SPIClass(FSPI);
};

void MotionDetection::begin(void){
    pinMode(34,OUTPUT);
    digitalWrite(34,HIGH);
    handler->begin(36,37,35,34);
     
    handler->beginTransaction(SPISettings(frequency,SPI_MSBFIRST,SPI_MODE0)); 
    digitalWrite(34,LOW);
    // set Accel and Gyroscop to Low Noise
    handler->transfer(PWR_MGMT0);
    handler->transfer(0x1F);
    //busy Wait for startup 
    delayMicroseconds(250);
    //set Gyroconfig
    handler->transfer(0x20);
    handler->transfer(0x25);
    //set Gyro Filter
    handler->transfer(0x23);
    handler->transfer(0x37);
    //Disable FIFO Bypass
    handler->transfer(FIFO_CONFIG1);
    handler->transfer(0x00);
    digitalWrite(34,HIGH);
    handler->endTransaction();
    this->writeToRegisterBank(MREG1,FIFO_CONFIG5,0x23);
    //Enable Gyro and Acceldata in FIFO

};
void MotionDetection::end(void){
    handler->beginTransaction(SPISettings(frequency,SPI_MSBFIRST,SPI_MODE0));
    digitalWrite(34,LOW);
    handler->transfer(cmdWrite(PWR_MGMT0));
    //turn Accel and Gyroscope off
    handler->transfer(0x00);
    digitalWrite(34,HIGH);
    handler->end();
};
IMUResult MotionDetection::getAcceleration(){
    IMUResult result;
    result.x = readRegister(ACCEL_DATA_X_HIGH)<<8;
    result.x |= readRegister(ACCEL_DATA_X_LOW);
    result.y = readRegister(ACCEL_DATA_Y_HIGH)<<8;
    result.y |= readRegister(ACCEL_DATA_Y_LOW);
    result.z = readRegister(ACCEL_DATA_Z_HIGH)<<8;
    result.z |= readRegister(ACCEL_DATA_Z_LOW);
    return result;
};
IMUResult MotionDetection::getRotation(){
    IMUResult result;
    result.x = readRegister(GYRO_DATA_X_HIGH) <<8;
    result.x |= readRegister(GYRO_DATA_X_LOW);
    result.y = readRegister(GYRO_DATA_Y_HIGH)<<8;
    result.y |= readRegister(GYRO_DATA_Y_LOW);
    result.z = readRegister(GYRO_DATA_Z_HIGH)<<8;
    result.z |= readRegister(GYRO_DATA_Z_LOW);
    return result;
};
float MotionDetection::getTemperature(){
    int16_t raw_temperatur = readRegister(REG_TEMP_HIGH)<<8;
    raw_temperatur |= readRegister(REG_TEMP_LOW); 
    return raw_temperatur/128 +25;
};

int8_t MotionDetection::getWhoAmI(){
    return readRegister(WHO_AM_I);    
};

uint8_t MotionDetection::cmdRead(uint8_t reg){
    return (CMD_READ | (reg & ADDR_MASK));
};
uint8_t MotionDetection::cmdWrite(uint8_t reg){
    return (CMD_WRITE | (reg & ADDR_MASK));
};

uint8_t MotionDetection::readRegister(uint8_t reg){
    handler->beginTransaction(SPISettings(frequency,SPI_MSBFIRST,SPI_MODE0));
    digitalWrite(34,LOW);
    uint8_t result;
    result = handler->transfer(cmdRead(reg));
    result = handler->transfer(0x00);
     digitalWrite(34,HIGH);
    handler->endTransaction(); 
    return result;
};

uint8_t MotionDetection::readFromRegisterBank(registerBank bank,uint8_t reg){
    uint8_t result = 0;
    handler->beginTransaction(SPISettings(frequency,SPI_MSBFIRST,SPI_MODE0));
    digitalWrite(34,LOW);
    switch(bank){
        case(MREG1):
            result = handler->transfer(BLK_SEL_R);
            result = handler->transfer(0x00);
            break;
        case(MREG2):
            result = handler->transfer(BLK_SEL_R);
            result = handler->transfer(0x28);
            break;
        case(MREG3):
            result = handler->transfer(BLK_SEL_R);
            result = handler->transfer(0x50);
            break;
    }
    result = handler->transfer(MADDR_R);
    result = handler->transfer(reg);
    digitalWrite(34,HIGH);
    handler->endTransaction(); 
    delayMicroseconds(10);
     result=this->readRegister(M_R);
    delayMicroseconds(10);
    this->resetRegisterBankAccess();
    return result;
};

void MotionDetection::writeToRegisterBank(registerBank bank, uint8_t reg, uint8_t value){
    while((this->readRegister(MCLK_RDY))&0x08!=0x08){
        Serial.println("CLK not rdy");
        delay(100);
    }
    Serial.print("MADDR_W: ");
    Serial.println(readRegister(MADDR_W)); 
    handler->beginTransaction(SPISettings(frequency,SPI_MSBFIRST,SPI_MODE0));
    digitalWrite(34,LOW);
    switch(bank){
        case(MREG1):
            handler->transfer(BLK_SEL_W);
            handler->transfer(0x00);
            break;
        case(MREG2):
            handler->transfer(BLK_SEL_W);
            handler->transfer(0x28);
            break;
        case(MREG3):
            handler->transfer(BLK_SEL_W);
            handler->transfer(0x50);
            break;
    }
    handler->transfer(MADDR_W);
    handler->transfer(reg);
    //handler->transfer(reg);
    handler->transfer(M_W);
    handler->transfer(value);
    digitalWrite(34,HIGH);
    handler->endTransaction();
    Serial.print("MADDR_W: ");
    Serial.println(readRegister(MADDR_W));
    delayMicroseconds(10);
    this->resetRegisterBankAccess();

};

void MotionDetection::resetRegisterBankAccess(){
    handler->beginTransaction(SPISettings(frequency,SPI_MSBFIRST,SPI_MODE0));
    digitalWrite(34,LOW);
    handler->transfer(BLK_SEL_R);
    handler->transfer(0x00);
    handler->transfer(BLK_SEL_W);
    handler->transfer(0x00);
    handler->transfer(MADDR_R);
    handler->transfer(0x00);
    handler->transfer(MADDR_W);
    handler->transfer(0x00);
    digitalWrite(34,HIGH);
    handler->endTransaction();
};

void MotionDetection::testFIFO(){
    this->writeToRegisterBank(MREG1,FIFO_CONFIG5,0x23);
    //Serial.println(this->readFromRegisterBank(MREG1,0x47));
    //uint16_t fifocount = this->readRegister(FIFO_COUNTH)<<8;
    //fifocount |= this->readRegister(FIFO_COUNTL);
    //Serial.print("FiFo Count: ");
    //Serial.println(fifocount);

    Serial.print("INT_CONFIG1: ");
    Serial.println(readFromRegisterBank(MREG1,0x05));
    Serial.print("FiFoData: ");
    Serial.println(readRegister(0x3F));
};